
      subroutine svd_ml(nobs,amp,dist,stcoid,evid,ref,
     &  magscale,magscale_err,siteterm,siteterm_err,
     &  sourceterm,sourceterm_err,err)
c
c input:
c
c    nobs - number of observations
c    amp - amplitude in nm to calculate Ml
c    dist - event-station distance in km
c    stcoid - stream index
c    evid - event index
c    ref - distance reference: 1=dist, 2=amp, 3=mag
c
c output:
c
c    mag = log (amp) + a log(dist/refdist) + b (dist-refdist) +c
c                     (1)                   (2)               (3)
c    magscale_err - magscale error 
c    siteterm - site term
c    siteterm_err - site term error
c    sourceterm - source term
c    sourceterm_err - source term error
c
c    err - error: 0 if ok, -1 if not
c    
c
      implicit none
      
      integer nobs
      real amp(*),dist(*)
      integer evid(*),stcoid(*)
      real magscale(*),magscale_err(*)
      real sourceterm(*),sourceterm_err(*)
      real siteterm(*),siteterm_err(*)

      integer max_x,max_y             ! array dimensions for inversion
      parameter (max_x=2000)          ! size of model vector
      parameter (max_y=10*max_x)         ! number of observations
      real data_vector(max_y)         ! amplitude vector
      real kernel_matrix(max_y,max_x) ! kernel matrix
      real model_vector(max_x)        ! model vector
      real cvm(max_x,max_x)           ! covariance matrix
      integer maxsta                  ! maximum number of stations array dimensions
      parameter(maxsta=300)
      integer maxst                   ! actual maximum number of stations
      real v(max_x,max_x),w(max_y)    ! matrices used with SVD
      real thresh,wmax,tol            ! used when editing the w vector
      parameter (tol=1.E-5)
      integer mp,np                   ! number of cols and rows in matrices
      integer i,j,k                   ! counters
      integer maxev                   ! maximum event id
      integer err
      real ref(3)                     ! reference, 1:ref dist, 2=amp (WA mm), 3=mag
      real wagain
      parameter (wagain=2080.)
      logical flag
      logical inversion_test

      inversion_test=.false.

c
c init variable and arrays
c

c ml=3 gives 10 mmWA at 100km distance Richter                         
c ml=3 gives 17 mmWA at 17km distance (Hutton and Boore, 1987)
      if (ref(1).eq.0.) then
        ref(1)=100.
        ref(2)=1.
        ref(3)=3.
      endif
      maxst=0
      maxev=0
      err=0
c
c init 
c
      do i=1,nobs+3
        data_vector(i)=0.
        do j=1,max_x
          if (i.eq.1) model_vector(j)=0.
          kernel_matrix(i,j)=0.
        enddo
      enddo
      if (nobs.gt.max_y) then
        err=-1
      endif

c ML = log A + a log(dist/refdist) + b (dist-refdist) + c + S
c log A = ML - a log(dist/refdist) - b (dist-refdist) - c - S

c
c fill kernel matrix
c
      do i=1,nobs
        if (amp(i).ne.0.) then
          if (stcoid(i).gt.maxst) maxst=stcoid(i)
          if (evid(i).gt.maxev) maxev=evid(i)

          data_vector(i)=alog10(amp(i))              ! data

          kernel_matrix(i,1)=-alog10(dist(i)/ref(1)) ! log distance
          kernel_matrix(i,2)=-(dist(i)-ref(1))       ! distance
ccc taken out lo 14/01/2010
c          kernel_matrix(i,3)=-1.                     ! constant
          kernel_matrix(i,3+stcoid(i))=-1.           ! station term
          kernel_matrix(i,3+maxsta+evid(i))=1.       ! event size term, ML
          if (3+maxsta+evid(i).gt.max_x) then
            write(*,*) ' dimensions exceeded, stop '
            stop
          endif
c          write(25,*) i,stcoid(i),evid(i)
        endif
      enddo

c
c invert for Ml scale
c
ccc        mp=nobs+3
        mp=nobs+3
c
c add constraint for constant added to source term
c at refdist log A = ML - c
c
        maxev=maxev+1
        data_vector(mp-1)=alog10(ref(2)/wagain*1E6)-ref(3) ! log A - ML
        kernel_matrix(mp-1,1)=0.
        kernel_matrix(mp-1,2)=0.    
        kernel_matrix(mp-1,3)=-1.                          ! -c
c
c add reference magnitude to kernel
c

ccc lo 11/01/2010 this should not be needed
        data_vector(mp-2)=ref(3)
        kernel_matrix(mp-2,3+maxsta+maxev)=1.

c
c add constraint for all site terms to sum up to 0
c
        data_vector(mp)=0.
        do i=4,3+maxst
          kernel_matrix(mp,i)=1
        enddo
c
c remove empty columns from kernel matrix
c
        do j=1,maxev
          do i=1,mp
            kernel_matrix(i,j+3+maxst)=
     &           kernel_matrix(i,j+3+maxsta)
            kernel_matrix(i,j+3+maxsta)=0.
          enddo
        enddo
        np=3+maxst+maxev
        write(*,*) np,maxst,maxev
        write(*,*) ' number of equations ',mp
        write(*,*) ' number of model parameters ',np
        if (np.gt.mp) then
          write(*,*) ' too few data, stop '
          stop
        endif
c
c check that no empty columns
c
        flag=.false.
        do i=1,np
          k=0
          do j=1,mp
            if (kernel_matrix(j,i).ne.0.) then
              flag=.true.
              k=k+1
            endif
          enddo
          write(24,*) i,k
        enddo
        if (.not.flag) then
          write(*,*) ' empty column, stop '
          stop
        endif
c
c write out lin equations
c
        do i=1,mp
          write(23,*) data_vector(i),' = ',(kernel_matrix(i,j),j=1,np)
        enddo
        write(*,*) ' calling svdcmpx '
        call svdcmpx(kernel_matrix,mp,np,max_y,max_x,w,v)
c
c edit w, remove small values
c
         wmax=0.
         do j=1,max_x
           if(w(j).gt.wmax)wmax=w(j)
         enddo
         thresh=tol*wmax
         do j=1,max_x
           if (w(j).lt.thresh)w(j)=0.
         enddo

c
c call svbksb
c
        write(*,*) ' calling svbksbx '
        call svbksbx(kernel_matrix,w,v,mp,np,max_y,max_x,
     *      data_vector,model_vector)

c
c call svdvar to get variance
c
        write(*,*) ' calling svdvar '
        call svdvar(v,np,max_x,w,cvm,max_x)
c
c set output
c
      magscale(1)=model_vector(1)
      magscale(2)=model_vector(2)
      magscale(3)=model_vector(3)

      magscale_err(1)=sqrt(cvm(1,1))
      magscale_err(2)=sqrt(cvm(2,2))
      magscale_err(3)=0.

      do i=1,maxst
        siteterm(i)=model_vector(i+3)
        siteterm_err(i)=sqrt(cvm(i+3,i+3))
      enddo
      do i=1,maxev-1
        sourceterm(i)=model_vector(i+3+maxst)
        sourceterm_err(i)=sqrt(cvm(i+3+maxst,i+3+maxst))
      enddo

      return
      end

